/*
The GPLv3 License (GPLv3)

Copyright (c) 2022 Akos Horvath

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CLIENT_H
#define CLIENT_H

#include <X11/X.h>
#include <X11/Xlib.h>
#include <stdbool.h>

typedef struct Wm Wm;
typedef struct Workspace Workspace;
typedef struct Monitor Monitor;
typedef struct Client Client;

struct Client {
    int x;
    int y;
    int w;
    int h;
    Workspace *ws;
    bool hidden;
    Monitor *m;
    Window window;
    Window frame;
    Client *prev;
    Client *next;
    char *name;
    bool focused;
    bool has_border;
    bool is_floating;
    pid_t pid;
    bool is_pid_set;
};

/**
 * Client functions
 */

/**
 * Converts the position of `c` to a `XWindowChanges` structure.
 */
XWindowChanges wm_client_to_xwchanges(Client *c);

/**
 * Returns the client associated with `w`.
 * Returns NULL if it could not find the client.
 */
Client* wm_client_find(Wm* wm, Window w);

/**
 * Initializes a new Client.
 */
Client* wm_client_create(Wm *wm, Window w);

void wm_client_handle_window_types(Wm *wm, Client *c, XMapRequestEvent e);

/**
 * Unmaps `c->window` and `c->frame`.
 */
void wm_client_hide(Wm *wm, Client *c);

/**
 * Maps `c->window` and `c->frame`.
 */
void wm_client_show(Wm* wm, Client *c);

/**
 * Focuses `c`.
 */
void wm_client_focus(Wm* wm, Client *c);

/**
 * Focuses the client in the direction `dir`, relative to the current focused
 * client.
 */
void wm_client_focus_dir(Wm* wm, Client *c, int dir);

/**
 * Swaps the client in the direction `dir`, relative to the current focused
 * client.
 */
void wm_client_swap_dir(Wm* wm, Client *c, int dir);

/**
 * Frees `c`, and its dynamically allocated fields.
 */
void wm_client_free(Wm *wm, Client *c);

/**
 * Destroys the window and frame associated with `c`, frees `c`, and recalculates
 * the layout.
 */
void wm_client_kill(Wm *wm, Client *c);

/**
 * Sets the window property with the atom of `name`, with the value `data`.
 */
void wm_client_set_atom(Wm *wm, Client *c, const char *name, const unsigned char *data,
                        Atom type, int nelements);

/**
 * Reads the window poperty with the atom of `name`, and returns the value
 * in `atom_ret`, and returns the number of read items in `nitems_ret`.
 */
Atom wm_client_get_atom(Wm *wm, Client *c, const char *name, unsigned char **atom_ret,
                        unsigned long *nitems_ret);


/**
 * Finds the client in the direction `dir` relative to `c`.
 */
Client* wm_client_get_dir_rel_c(Client *c, int dir);

/**
 * Returns the currently focused client.
 */
Client* wm_client_get_focused(Wm* wm);

/**
 * Draws border for client `c`.
 */
void wm_client_border(Wm* wm, Client *c);

/**
 * Draws focused border for client `c`.
 */
void wm_client_border_focused(Wm* wm, Client *c);

/**
 * Draws borders for all clients in `m`.
 */
void wm_monitor_clients_border(Wm* wm, Monitor *m);

/**
 * Returns true if `c` is focused, false if `c` is not focused.
 */
bool wm_client_is_focused(Wm* wm, Client *c);

/**
 * Draws server-side decorations for `c`.
 */
void wm_client_draw_frame(Wm *wm, Client *c);

/**
 * Draws server-side decorations for all clients in `m`.
 */
void wm_monitor_draw_all_clients_frame(Wm* wm, Monitor *m);

#endif
